#include "gradImage.h"


int compare( const void *elem1, const void *elem2 )
{
	return (*(int *)elem1>*(int *)elem2);
}
BYTE ImageGetPixel(BYTE * image, int i, int j, int width, int height)
{
	int pos = j * width + i;
	return image[pos];
}
double ImageGetPixelf(double * gradHx, int i, int j, int width, int height)
{
	int pos = j * width + i;
	return gradHx[pos];
}

void ImageSetPixelf(double * divG, int i, int j, double val, int width, int height)
{
	int pos = j * width + i;
	divG[pos] = val;
}
void imageGradient(BYTE *image, double *gradHx, double *gradHy, int width, int height)
{
  int i, j;

  double gradX, gradY;
  
  for (i=0; i < width; i++) {
	  for (j=0; j < height; j++) {
		  int pos = j * width + i;
		  if (i != width - 1){
			  gradX = ImageGetPixel(image, i + 1, j, width, height) - ImageGetPixel(image, i, j, width, height);
		  }
		  else gradX = 0;
		  if(j != height - 1){
			  gradY = ImageGetPixel(image, i, j+1, width, height) - ImageGetPixel(image, i, j, width, height);
		  }
		  else gradY = 0;
		  
		  //set mask
		  gradHx[pos] = gradX; gradHy[pos] = gradY;		  
	  }
  }
}
void MergeGradient(double * gradHx1, double * gradHy1, double * gradHx2, double * gradHy2, 
				   double * gradHxfinal, double * gradHyfinal, BYTE * mask, int width, int height)
{
	//merge the gradient by the mask (not necessarily binary)
	for (int i = 0; i < width; i++){
		for(int j = 0; j < height; j++){
			int pos = width * j + i;
			double rate = mask[pos]/255.0;
			gradHxfinal[pos] = gradHx1[pos] * rate + gradHx2[pos] * (1-rate);
			gradHyfinal[pos] = gradHy1[pos] * rate + gradHy2[pos] * (1-rate);
		}
	}
}

void imageGradientTodivG(double *gradHx, double *gradHy, double *divG, int width, int height)
{
	
	int i,j;
	double gradSumX, gradSumY;

	for (i=0; i < width; i++) {
		for (j=0; j < height; j++) {
			int pos = j * width + i;
			if (i == 0) gradSumX = 0;
			else gradSumX = (double) (ImageGetPixelf(gradHx, i, j, width, height) - ImageGetPixelf(gradHx, i-1, j, width, height));
			if (j == 0) gradSumY = 0;
			else gradSumY = (double) (ImageGetPixelf(gradHy, i, j, width, height) - ImageGetPixelf(gradHy, i, j-1, width, height));
			
			ImageSetPixelf(divG, i, j, (gradSumX + gradSumY)/255.0, width, height) ; 
		}
	}
}

void imageNormalize(double ** u, double **out, int width, int height)
{
	int i,j;

	double max = -10000000.0, min = 1000000000.0;;

	
	for (i=1; i <= width; i++) {
		for (j=1; j <= height; j++) {
			if (u[i][j] > max) max = u[i][j];
			if (u[i][j] < min) min = u[i][j];
		}
	}
	for (i=1; i <= width; i++) {
		for (j=1; j <= height; j++) {
			out[i][j] = (u[i][j] - min) / (max - min) * 255;
		}
	}
}

